/* Tray for the Mini Reels, version 2

   The mini-reel is a smaller version of the standard 7-inch reel.
   This design is a tray for these mini-reels.

   Copyright 2022-2023, CompuPhase, Thiadmer Riemersma
   This work is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License
   (https://creativecommons.org/licenses/by-sa/4.0/).
 */

/* [Options] */

// The length of the tray
Tray_Length = 190;

// The standard mini-reel is 100mm (if you may reels with other dimensions, this value can be adjusted). Note: Diameter_Tolerance is added to this value.
Reel_Diameter = 100;

// Slant of the top, in degrees
Top_Slant = 36;

// The tolerance (or slack) for the cylindrical cutout of the tray
Diameter_Tolerance = 0.5;

// Thickness of the end panels and divider panels between compartments
Panel_Thickness = 1.25;

// If true, there is a closing panel at the left side.
End_Panel_Left = true;

// If true, there is a closing panel at the right side.
End_Panel_Right = true;

// Number of slots for separator panels
Separator_Panels = 14;

// Tolerance between the separator panels en the slots for these panels
Panel_Tolerance = 0.2;

// Diameter of alignment pins, to help glue multiple trays together
Alignment_Pin_Diameter = 3; // 0.1

/* [Hidden] */

epsilon = 0.05;

InsideDiameter = Reel_Diameter + Diameter_Tolerance;
OutsideDiameter = InsideDiameter + 2*Panel_Thickness;
Avg_Height = InsideDiameter * 0.4;                /* average tray height (if the top slant is 0 degrees, this is the height) */
Max_Height = InsideDiameter/2 + Panel_Thickness;  /* tray does not exceed half of the cylindrical cutout */
Tray_Width = InsideDiameter + 2*Panel_Thickness;  /* total width is also a function of the reel diameter */
Slot_Sections = ceil(Tray_Length / 100);          /* number of oblong slots in the the bottom */


include <bezier.scad>

/* helper functions to calculate intersection of line with circle
   see https://mathworld.wolfram.com/Circle-LineIntersection.html */
function sq(v) = v * v;
pseudo_org = (Top_Slant > 0) ? -(Avg_Height / tan(Top_Slant)) : -1.0E200;
function delta_x(P, Q) = Q[0] - P[0];
function delta_y(P, Q) = Q[1] - P[1];
function delta_r(P, Q) = sqrt( sq(delta_x(P, Q)) + sq(delta_y(P, Q)) );
function cross_D(P, Q) = P[0] * Q[1] - Q[0] * P[1];
function discriminant(R, P, Q) = sq(R) * sq(delta_r(P, Q)) - sq(cross_D(P, Q));
function line_circle_intersect(R, P, Q) = [
            (cross_D(P, Q) * delta_y(P, Q) + sign(delta_y(P, Q)) * delta_x(P, Q) * sqrt(discriminant(R, P, Q))) / sq(delta_r(P, Q)),
            (-cross_D(P, Q) * delta_x(P, Q) + abs(delta_y(P, Q)) * sqrt(discriminant(R, P, Q))) / sq(delta_r(P, Q))
         ];
SlopeCylinderJoint = line_circle_intersect(InsideDiameter/2, [pseudo_org, 0], [0, OutsideDiameter]);

module cutoff_curve(width, slant, maxheight) {
  Lx = -width/2 - epsilon;
  Ly = Lx * tan(slant);
  Tx = width/2 + epsilon;
  Ty = maxheight;
  Cx = (slant > 0) ? Lx + (Ty - Ly) / tan(slant) : 999999;
  if (Lx <= Cx && Cx <= Tx) {
    difference() {
      polygon([[0,epsilon],[Tx-Lx,Ty-Ly+epsilon],[Tx-Lx,width-Ly],[0,width-Ly]]);
      BezierPolygon([[0,0],[Cx-Lx,Ty-Ly],[Cx-Lx,Ty-Ly],[Tx-Lx,Ty-Ly]], 20);
    }
  } else {
    square([Tx - Lx, width]);
  }
}

module top_cutoff(length, width, slant, maxheight) {
  rotate([0, 90, 0])
    rotate([0, 0, 90])
      linear_extrude(length)
        cutoff_curve(width, slant, maxheight);
}

module panel_curve(width, slant, avgheight, maxheight, tolerance) {
  Lx = -width/2;
  Ly = Lx * tan(slant);
  Tx = width/2;
  Ty = maxheight - avgheight;
  Cx = (slant > 0) ? Lx + (Ty - Ly) / tan(slant) : 999999;
  union() {
    intersection() {
      union() {
        if (Lx <= Cx && Cx <= Tx)
          BezierPolygon([[0,0],[Cx-Lx,Ty-Ly],[Cx-Lx,Ty-Ly],[Tx-Lx,Ty-Ly]], 20);
        polygon([[0,0],[Tx-Lx,0],[Tx-Lx,Ty-Ly+epsilon],[0,epsilon]]);
      }
      translate([width/2, width/2 - 2*Panel_Thickness])
      circle(d = InsideDiameter, $fn = 100);
    }
    leg_length = 16;
    leg_width = 10;
    translate([width - Panel_Thickness - leg_width, (Ty-Ly) - leg_length - tolerance])
      square([leg_width + tolerance, leg_length + tolerance]);
    translate([width/2 + SlopeCylinderJoint[0] + 2*Panel_Thickness - tolerance, 0])
      square([leg_width + 2*tolerance, 12.5]);
  }
}

module separator_panel(tolerance) {
  linear_extrude(Panel_Thickness + tolerance)
    panel_curve(Tray_Width, Top_Slant, Avg_Height, Max_Height, tolerance);
}

module basic_block() {
  difference() {
    union() {
      /* solid cylinder, from which the cylindrical reel compartment will be cut out */
      translate([0, 0, OutsideDiameter/2])
        rotate([0, 90, 0])
          cylinder(d=OutsideDiameter, h=Tray_Length, $fn=100);
      /* fully filled block below the part where the slope is shallow */
      translate([0, -Tray_Width/4, 0])
        cube([Tray_Length, Tray_Width/2, Avg_Height - epsilon]);
      /* panels at the left/right ends */
      translate([0, -Tray_Width/2, 0])
        cube([Panel_Thickness, Tray_Width, Max_Height]);
      translate([Tray_Length - Panel_Thickness, -Tray_Width/2, 0])
        cube([Panel_Thickness, Tray_Width, Max_Height]);
      /* panels at the front/back sides */
      z_low_end = Avg_Height - (Tray_Width/2)*tan(Top_Slant) - Panel_Thickness;
      assert(z_low_end > 0);
      translate([0, -Tray_Width/2, 0])
        cube([Tray_Length, Panel_Thickness, z_low_end + Panel_Thickness]);
      translate([0, Tray_Width/2 - 2*Panel_Thickness, 0])
        cube([Tray_Length, 2*Panel_Thickness, Max_Height]);
      /* ridges at the bottom for extra stiffness */
      Ridge = 6;
      translate([0, -Tray_Width/2, 0])
        cube([Tray_Length, Ridge, Panel_Thickness]);
      translate([0, Tray_Width/2 - Ridge, 0])
        cube([Tray_Length, Ridge, Panel_Thickness]);
      for (idx = [1 : Slot_Sections - 1]) {
        offs_slot = idx * (Tray_Length / Slot_Sections);
        translate([offs_slot -Ridge/2, -Tray_Width/2, 0])
          cube([Ridge, Tray_Width, Panel_Thickness]);
      }
      /* slanted connecting ridge at the front side (low side) */
      translate([0, Panel_Thickness/2 - Tray_Width/2, z_low_end])
        rotate([Top_Slant, 0, 0])
          cube([Tray_Length, Tray_Width/2, Panel_Thickness]);
      /* another (vertical) ridge just behind the slanted ridge, for the separator panels to slide in */
      translate([0, SlopeCylinderJoint[0] + Panel_Thickness, 0])
        cube([Tray_Length, 2*Panel_Thickness, Tray_Width/2]);
      /* horizontal connecting ridge at the back side (high side) */
      z_high_end = Avg_Height + (Tray_Width/2)*tan(Top_Slant) - Panel_Thickness;
      if (z_high_end > Max_Height)
        translate([0, 0, Max_Height - Panel_Thickness])
          cube([Tray_Length, Tray_Width/2 + epsilon, Panel_Thickness]);
    }

    /* cut off slanted top */
    cutblk_z = Avg_Height - (Tray_Width/2)*tan(Top_Slant);
    translate([-Panel_Thickness - epsilon, -Tray_Width/2 - epsilon, cutblk_z])
      top_cutoff(Tray_Length + 2*Panel_Thickness + 2*epsilon, Tray_Width, Top_Slant, Max_Height - Avg_Height);
  }
}

module cylinder_cutout(length) {
  cylinder(d=InsideDiameter, h=length, $fn=100);
}

module oblong_slot(length, width, height) {
  diameter = min(width, length);
  radius = diameter / 2;
  offs_x = (length > width) ? length / 2 - radius : 0;
  offs_y = (length > width) ? 0 : width / 2 - radius;

  union() {
    translate([-offs_x, -offs_y, 0])
      cylinder(d = diameter, h = height, $fn=50);
    translate([offs_x, offs_y, 0])
      cylinder(d = diameter, h = height, $fn=50);

    if (length > width)
      translate([-offs_x, -width / 2, 0])
        cube([length - diameter, width, height]);
    else
      translate([-length / 2, -offs_y, 0])
        cube([length, width - diameter, height]);
  }
}

module tray() {
  difference() {
    /* basic block */
    basic_block();

    /* cylindrical cut-out for the reels */
    end_panels = (End_Panel_Left && End_Panel_Right) ? 2
                 : (End_Panel_Left || End_Panel_Right) ? 1 : 0;
    cutout_length = (Tray_Length - end_panels * Panel_Thickness) + (2 - end_panels) * epsilon;
    offs_x = End_Panel_Left ? Panel_Thickness : -epsilon;
    translate([offs_x, 0, InsideDiameter/2 + Panel_Thickness])
      rotate([0, 90, 0])
        cylinder_cutout(cutout_length);

    /* cut out slots in the middle, because it is too thin to be useful */
    slot_step = cutout_length / Slot_Sections;
    slot_length = slot_step - 10;
    slot_width = 20;
    if (slot_length > 25) {
      for (idx = [0 : Slot_Sections - 1]) {
        offs_slot = idx * slot_step;
        translate([offs_slot + slot_step/2, 0, -epsilon])
          oblong_slot(slot_length - 10, slot_width, 2 * Panel_Thickness);
      }
    }

    /* slots for extra separator panels */
    inside_length = Tray_Length - (end_panels == 0 ? 2 : 1) * Panel_Thickness - epsilon;
    panel_step = inside_length / (Separator_Panels + 1);
    slot_count = End_Panel_Right ? Separator_Panels : Separator_Panels + 1;
    for (idx = [1 : slot_count]) {
      translate([idx * panel_step, -Tray_Width/2, 2*Panel_Thickness])
        rotate([0, 90, 0])
          rotate([0, 0, 90])
           separator_panel(Panel_Tolerance);
    }

    /* alignment pins */
    if (!End_Panel_Left) {
      translate([-epsilon, -Tray_Width*0.37, 2*Alignment_Pin_Diameter])
        rotate([0, 90, 0])
          cylinder(d = Alignment_Pin_Diameter, h = Panel_Thickness + 2*epsilon, $fn = 20);
      translate([-epsilon, Tray_Width*0.37, 2*Alignment_Pin_Diameter])
        rotate([0, 90, 0])
          cylinder(d = Alignment_Pin_Diameter, h = Panel_Thickness + 2*epsilon, $fn = 20);
    }
    if (!End_Panel_Right) {
      translate([Tray_Length - Panel_Thickness - epsilon, -Tray_Width*0.37, 2*Alignment_Pin_Diameter])
        rotate([0, 90, 0])
          cylinder(d = Alignment_Pin_Diameter, h = Panel_Thickness + 2*epsilon, $fn = 20);
      translate([Tray_Length - Panel_Thickness - epsilon, Tray_Width*0.37, 2*Alignment_Pin_Diameter])
        rotate([0, 90, 0])
          cylinder(d = Alignment_Pin_Diameter, h = Panel_Thickness + 2*epsilon, $fn = 20);
    }
  }
}

tray();
//separator_panel(0);